package org.netkernel.cp;

import java.io.StringWriter;
import java.text.NumberFormat;

import org.netkernel.layer0.nkf.INKFLocale;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.representation.impl.HDSPredicateFactory;
import org.netkernel.layer0.util.Utils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.xml.xda.DOMXDA;
import org.netkernel.xml.xda.IXDAReadOnly;
import org.w3c.dom.Document;

public class WidgetsAccessor extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		String id=aContext.getThisRequest().getArgumentValue("id");
		IHDSNodeList config=aContext.source("netkernel:/config",IHDSNode.class).getNodes("/config/item");
		try
		{	if (id.equals("Instance"))
			{	onInstance(config,aContext);
			}
			else if (id.equals("Uptime"))
			{	onUptime(config,aContext);
			}
			else if (id.equals("Product"))
			{	onProduct(config,aContext);
			}
			else if (id.equals("Version"))
			{	onVersion(config,aContext);
			}
			else if (id.equals("InfoBox"))
			{	onInfoBox(aContext,false);
			}
			else if (id.equals("InfoBoxOpen"))
			{	onInfoBox(aContext,true);
			}
		}
		catch (Exception e)
		{	aContext.logRaw(INKFLocale.LEVEL_WARNING, Utils.throwableToString(e));
		}
	}
	
	private void onInfoBox(INKFRequestContext aContext, boolean aShow) throws Exception
	{
		IXDAReadOnly content = aContext.source("arg:content",IXDAReadOnly.class);
		Document template = aContext.source("res:/org/netkernel/cp/infobox.xml",Document.class);
		DOMXDA xda=new DOMXDA(template,true);
		xda.append(content, "/", "/div/table/tr/td[2]");
		if (aShow)
		{	xda.delete("/div/@style");
		}
		aContext.createResponseFrom(xda);
	}
	
	private void onInstance(IHDSNodeList aConfig, INKFRequestContext aContext) throws Exception
	{
		String instance=getConfigValue(aConfig,"netkernel.instance.identifier");
		String result="<span>"+instance+"</span>";
		INKFResponse resp=aContext.createResponseFrom(result);
		resp.setExpiry(INKFResponse.EXPIRY_NEVER);
	}
	
	private void onUptime(IHDSNodeList aConfig, INKFRequestContext aContext) throws Exception
	{
		long bootTime=Long.parseLong(getConfigValue(aConfig,"netkernel.boot.time"));
		long uptimePeriod=System.currentTimeMillis()-bootTime;
		String uptime=Utils.formatPeriod(uptimePeriod, false);
		
		String result="<span>"+uptime+"</span>";
		INKFResponse resp=aContext.createResponseFrom(result);
		resp.setExpiry(INKFResponse.EXPIRY_MIN_CONSTANT_DEPENDENT, System.currentTimeMillis()+60000L);
	}

	private void onVersion(IHDSNodeList aConfig, INKFRequestContext aContext) throws Exception
	{
		String major=getConfigValue(aConfig,"netkernel.instance.version.major");
		String minor=getConfigValue(aConfig,"netkernel.instance.version.minor");
		String result="<span>"+major+" "+minor+"</span>";
		INKFResponse resp=aContext.createResponseFrom(result);
		resp.setExpiry(INKFResponse.EXPIRY_NEVER);
	}
	
	private void onProduct(IHDSNodeList aConfig, INKFRequestContext aContext) throws Exception
	{
		String product=getConfigValue(aConfig,"netkernel.instance.product");
		String result="<span>"+product+"</span>";
		INKFResponse resp=aContext.createResponseFrom(result);
		resp.setExpiry(INKFResponse.EXPIRY_NEVER);
	}

	private String getConfigValue(IHDSNodeList aConfig, String aName)
	{	try
		{	return aConfig.filter(HDSPredicateFactory.namedChildStringEquals("name", aName)).getFirstValue("value").toString();
		}
		catch (Exception e)
		{	return "["+aName+"]";
		}
	}
}